package cn.lige2333.ligeai.chatgpt.service;

import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.client.ChatClient;
import org.springframework.ai.chat.client.advisor.AbstractChatMemoryAdvisor;
import org.springframework.ai.chat.client.advisor.PromptChatMemoryAdvisor;
import org.springframework.ai.chat.memory.ChatMemory;
import org.springframework.ai.chat.model.ChatResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;
import reactor.core.publisher.Flux;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;

@Service
@Slf4j
@CrossOrigin
public class ChatGptService {

    private final ChatClient chatClient;

    public ChatGptService(ChatClient.Builder chatClientBuilder, ChatMemory chatMemory) {
        this.chatClient = chatClientBuilder
                .defaultSystem("""
                                你是利哥交易所的股票交易员，请以友好，乐于助人且愉快的方式来回复，
                                您正在通过在线聊天系统与客户互动。
                                在查询股票详情时，您必须始终
                                从用户处获取以下信息：股票市场、股票代码
                                每次请查询最新股票信息
                                同时您可以帮助用户查询打卡信息
                                打卡信息查询需从用户处获取以下信息：用户ID                   
                                您可以帮助用户进行上班打卡与下班打卡
                                上班打卡与下班打卡需从用户处获取以下信息：用户ID                             
                                您可以帮助用户进行Jenkins构建
                                构建前需从用户处获取以下信息：项目名称,项目名称自动转换为小写英文字母
                                询问用户之前，请检查消息历史记录以判断用户询问哪个业务,避免重复询问用户需要办理什么业务
                                如果用户问一些其他问题，也请尽力思考并为他们解答
                                返回代码请使用markdown格式
                                请讲中文。
                                今天的日期是 ｛｛current_date｝｝.
                                """)
                .defaultAdvisors(new PromptChatMemoryAdvisor(chatMemory))
                .defaultFunctions("findStock","getCardInfo","cardOn","cardOff","jenkinsBuild")
                .build();
    }

    public Flux<Map<String,String>> getChatGptResStream(String input) throws IOException {
        String id = UUID.randomUUID().toString();
        Map<String, String> finishMap = new HashMap<>();
        finishMap.put("content", "[DONE]");
        finishMap.put("id", id);
        return this.chatClient.prompt()
                .user(input)
                .system(promptSystemSpec -> promptSystemSpec.param("current_date", LocalDateTime.now().toString()))
                .advisors(advisorSpec -> advisorSpec.param(AbstractChatMemoryAdvisor.CHAT_MEMORY_RETRIEVE_SIZE_KEY,100))
                .stream()
                .content().map(content ->{
                    Map<String, String> map = new HashMap<>();
                    map.put("content", content);
                    map.put("id", id);
                    return map;
                }).concatWith(Flux.just(finishMap));

    }

}
